"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidationService = exports.PathBuffer = void 0;
const web3_1 = __importDefault(require("web3"));
const adm_zip_1 = __importDefault(require("adm-zip"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const CheckedContract_1 = __importDefault(require("./CheckedContract"));
/**
 * Regular expression matching metadata nested within another json.
 */
const NESTED_METADATA_REGEX = /"{\\"compiler\\":{\\"version\\".*?},\\"version\\":1}"/;
class PathBuffer {
    constructor(buffer, path) {
        this.buffer = buffer;
        this.path = path;
    }
}
exports.PathBuffer = PathBuffer;
class PathContent {
    constructor(content, path) {
        this.content = content;
        this.path = path;
    }
}
class ValidationService {
    /**
     * @param logger a custom logger that logs all errors; undefined or no logger provided turns the logging off
     */
    constructor(logger) {
        this.logger = logger;
    }
    checkPaths(paths, ignoring) {
        const files = [];
        paths.forEach(path => {
            if (fs_1.default.existsSync(path)) {
                this.traversePathRecursively(path, filePath => {
                    const fullPath = path_1.default.resolve(filePath);
                    const file = new PathBuffer(fs_1.default.readFileSync(filePath), fullPath);
                    files.push(file);
                });
            }
            else if (ignoring) {
                ignoring.push(path);
            }
        });
        return this.checkFiles(files);
    }
    checkFiles(files) {
        const inputFiles = this.findInputFiles(files);
        // const sanitizedFiles = this.sanitizeInputFiles(inputFiles);
        const parsedFiles = inputFiles.map(pathBuffer => new PathContent(pathBuffer.buffer.toString(), pathBuffer.path));
        const metadataFiles = this.findMetadataFiles(parsedFiles);
        const checkedContracts = [];
        const errorMsgMaterial = [];
        metadataFiles.forEach(metadata => {
            const { foundSources, missingSources, invalidSources } = this.rearrangeSources(metadata, parsedFiles);
            const checkedContract = new CheckedContract_1.default(metadata, foundSources, missingSources, invalidSources);
            checkedContracts.push(checkedContract);
            if (!checkedContract.isValid()) {
                errorMsgMaterial.push(checkedContract.info);
            }
        });
        if (errorMsgMaterial.length) {
            const msg = errorMsgMaterial.join("\n");
            if (this.logger)
                this.logger.error(msg);
        }
        return checkedContracts;
    }
    /**
     * Traverses the given files, unzipping any zip archive.
     *
     * @param files the array containing the files to be checked
     * @returns an array containing the provided files, with any zips being unzipped and returned
     */
    findInputFiles(files) {
        const inputFiles = [];
        for (const file of files) {
            if (this.isZip(file.buffer)) {
                this.unzip(file, files);
            }
            else {
                inputFiles.push(file);
            }
        }
        return inputFiles;
    }
    /**
     * Checks whether the provided file is in fact zipped.
     * @param file the buffered file to be checked
     * @returns true if the file is zipped; false otherwise
     */
    isZip(file) {
        try {
            new adm_zip_1.default(file);
            return true;
        }
        catch (err) {
            undefined;
        }
        return false;
    }
    /**
     * Unzips the provided file buffer to the provided array.
     *
     * @param zippedFile the buffer containin the zipped file to be unpacked
     * @param files the array to be filled with the content of the zip
     */
    unzip(zippedFile, files) {
        const timestamp = Date.now().toString();
        const tmpDir = `tmp-unzipped-${timestamp}`;
        new adm_zip_1.default(zippedFile.buffer).extractAllTo(tmpDir);
        this.traversePathRecursively(tmpDir, filePath => {
            const file = new PathBuffer(fs_1.default.readFileSync(filePath), zippedFile.path);
            files.push(file);
        });
        this.traversePathRecursively(tmpDir, fs_1.default.unlinkSync, fs_1.default.rmdirSync);
    }
    sanitizeInputFiles(files) {
        const sanitizedFiles = [];
        if (!files.length) {
            const msg = 'Unable to extract any files. Your request may be misformatted ' +
                'or missing some content.';
            if (this.logger)
                this.logger.error(msg);
            throw new Error(msg);
        }
        for (const file of files) {
            const content = file.buffer.toString();
            /*try {
                const val = JSON.parse(file.buffer.toString());
                const type = Object.prototype.toString.call(val);

                // JSON formatted metadata or Stringified metadata
                if (type === '[object Object]') {
                    content = JSON.stringify(val);
                } else if (Array.isArray(val)) {
                    content = JSON.stringify(val[0]);
                } else {
                    content = val;
                }

            } catch (err) {
                content = file.buffer.toString();          // Solidity files
            }*/
            sanitizedFiles.push(new PathContent(content, file.path));
        }
        return sanitizedFiles;
    }
    /**
     * Selects metadata files from an array of files that may include sources, etc
     * @param  {string[]} files
     * @return {string[]}         metadata
     */
    findMetadataFiles(files) {
        const metadataCollection = [];
        for (const file of files) {
            let metadata = this.extractMetadataFromString(file.content);
            if (!metadata) {
                const matchRes = file.content.match(NESTED_METADATA_REGEX);
                if (matchRes) {
                    metadata = this.extractMetadataFromString(matchRes[0]);
                }
            }
            if (metadata) {
                metadataCollection.push(metadata);
            }
        }
        if (!metadataCollection.length) {
            const msg = "Metadata file not found. Did you include \"metadata.json\"?";
            if (this.logger)
                this.logger.error(msg);
            throw new Error(msg);
        }
        return metadataCollection;
    }
    /**
     * Validates metadata content keccak hashes for all files and
     * returns mapping of file contents by file name
     * @param  {any}       metadata
     * @param  {string[]}  files    source files
     * @return foundSources, missingSources, invalidSources
     */
    rearrangeSources(metadata, files) {
        const foundSources = {};
        const missingSources = {};
        const invalidSources = {};
        const byHash = this.storeByHash(files);
        for (const fileName in metadata.sources) {
            const sourceInfo = metadata.sources[fileName];
            let file = new PathContent(undefined);
            file.content = sourceInfo.content;
            const hash = sourceInfo.keccak256;
            if (file.content) {
                if (web3_1.default.utils.keccak256(file.content) != hash) {
                    const msg = "The calculated and the provided hash values don't match.";
                    invalidSources[fileName] = msg;
                    continue;
                }
            }
            else {
                file = byHash.get(hash);
            }
            if (file && file.content) {
                foundSources[fileName] = file;
            }
            else {
                missingSources[fileName] = { keccak256: hash, urls: sourceInfo.urls };
            }
        }
        return { foundSources, missingSources, invalidSources };
    }
    /**
     * Generates a map of files indexed by the keccak hash of their content.
     *
     * @param  {string[]}  files Array containing sources.
     * @returns Map object that maps hash to PathContent.
     */
    storeByHash(files) {
        const byHash = new Map();
        for (const i in files) {
            const calculatedHash = web3_1.default.utils.keccak256(files[i].content);
            byHash.set(calculatedHash, files[i]);
        }
        return byHash;
    }
    extractMetadataFromString(file) {
        try {
            let obj = JSON.parse(file);
            if (this.isMetadata(obj)) {
                return obj;
            }
            // if the input string originates from a file where it was double encoded (e.g. truffle)
            obj = JSON.parse(obj);
            if (this.isMetadata(obj)) {
                return obj;
            }
        }
        catch (err) {
            undefined;
        }
        return null;
    }
    /**
     * A method that checks if the provided object was generated as a metadata file of a Solidity contract.
     * Current implementation is rather simplistic and may require further engineering.
     *
     * @param metadata the JSON to be checked
     * @returns true if the provided object is a Solidity metadata file; false otherwise
     */
    isMetadata(obj) {
        return (obj.language === "Solidity") &&
            !!obj.compiler;
    }
    /**
     * Applies the provided worker function to the provided path recursively.
     *
     * @param path the path to be traversed
     * @param worker the function to be applied on each file that is not a directory
     * @param afterDir the function to be applied on the directory after traversing its children
     */
    traversePathRecursively(path, worker, afterDirectory) {
        if (!fs_1.default.existsSync(path)) {
            const msg = `Encountered a nonexistent path: ${path}`;
            if (this.logger) {
                this.logger.error(msg);
            }
            throw new Error(msg);
        }
        const fileStat = fs_1.default.lstatSync(path);
        if (fileStat.isFile()) {
            worker(path);
        }
        else if (fileStat.isDirectory()) {
            fs_1.default.readdirSync(path).forEach(nestedName => {
                const nestedPath = path_1.default.join(path, nestedName);
                this.traversePathRecursively(nestedPath, worker, afterDirectory);
            });
            if (afterDirectory) {
                afterDirectory(path);
            }
        }
    }
}
exports.ValidationService = ValidationService;
//# sourceMappingURL=ValidationService.js.map